#include <QtTest>
#include <QVector>
#include <QHash>

class Rangefor_Test : public QObject
{
    Q_OBJECT

public:
    Rangefor_Test();

private Q_SLOTS:
    void rangeFor_Vector();
    void rangeFor_ConstVector();
    void iter_Vector();
    void iter_ConstVector();
    void forEach_Vector();
    void forEach_ConstVector();

    void rangeFor_Hash();
    void rangeFor_ConstHash();
    void forEach_Hash();
    void forEach_ConstHash();

private:
    const int containerSize;

};

Rangefor_Test::Rangefor_Test()
    : containerSize(5000000)
{
    qDebug() << "Container size: " << containerSize;
}

void Rangefor_Test::rangeFor_Vector()
{
    QVector<int> v(containerSize, 1);
    int sum = 0;
    QBENCHMARK {
        sum = 0;
        QVector<int> v2 = v;
        for (int i : v2) //non-const v2 detaches from v1
            sum += i;
    }
}

void Rangefor_Test::rangeFor_ConstVector()
{
    QVector<int> v(containerSize, 1);
    int sum = 0;
    QBENCHMARK {
        sum = 0;
        const QVector<int> cv = v;
        for (int i : cv) //no detaching necessary due to being const
            sum += i;
    }
}

void Rangefor_Test::iter_Vector()
{
    QVector<int> v(containerSize, 1);
    int sum = 0;
    QBENCHMARK {
        sum = 0;
        QVector<int> v2 = v;
        for (auto iter = v2.begin(); iter != v2.end(); ++iter)
            sum += *iter;
    }
}

void Rangefor_Test::iter_ConstVector()
{
    QVector<int> v(containerSize, 1);
    int sum = 0;
    QBENCHMARK {
        sum = 0;
        const QVector<int> v2 = v;
        for (auto iter = v2.begin(); iter != v2.end(); ++iter)
            sum += *iter;
    }
}

void Rangefor_Test::forEach_Vector()
{
    QVector<int> v(containerSize, 1);
    int sum = 0;
    QBENCHMARK {
        sum = 0;
        QVector<int> v2 = v;
        foreach (int i, v2)
            sum += i;
    }
}

void Rangefor_Test::forEach_ConstVector()
{
    QVector<int> v(containerSize, 1);
    int sum = 0;
    QBENCHMARK {
        sum = 0;
        const QVector<int> cv = v;
        foreach (int i, cv)
            sum += i;
    }
}

void Rangefor_Test::rangeFor_Hash()
{
    QHash<int, int> h;
    for (int i = 0; i < containerSize; ++i)
        h[i] = i;

    int sum = 0;
    QBENCHMARK {
        sum = 0;
        QHash<int, int> h2 = h;
        for (int i : h2)
            sum += i;
    }
}

void Rangefor_Test::rangeFor_ConstHash()
{
    QHash<int, int> h;
    for (int i = 0; i < containerSize; ++i)
        h[i] = i;

    int sum = 0;
    QBENCHMARK {
        sum = 0;
        const QHash<int, int> ch = h;
        for (int i : ch)
            sum += i;
    }
}

void Rangefor_Test::forEach_Hash()
{
    QHash<int, int> h;
    for (int i = 0; i < containerSize; ++i)
        h[i] = i;

    int sum = 0;
    QBENCHMARK {
        sum = 0;
        QHash<int, int> h2 = h;
        foreach (int i, h2)
            sum += i;
    }
}

void Rangefor_Test::forEach_ConstHash()
{
    QHash<int, int> h;
    for (int i = 0; i < containerSize; ++i)
        h[i] = i;

    int sum = 0;
    QBENCHMARK {
        sum = 0;
        const QHash<int, int> ch = h;
        foreach (int i, ch)
            sum += i;
    }
}

QTEST_APPLESS_MAIN(Rangefor_Test)

#include "tst_rangefor_test.moc"
